#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : ExplainerTabular
# @Time         : 2019-06-20 14:06
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

# TODO:
"""
https://github.com/microsoft/interpret
https://github.com/h2oai/mli-resources/blob/master/notebooks/dt_surrogate.ipynb
https://github.com/Jie-Yuan/lime
https://eli5.readthedocs.io/en/latest/blackbox/permutation_importance.html
https://github.com/slundberg/shap
"""


class ExplainerTabular(object):

    def __init__(self):
        pass
