#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'autokeras'
__author__ = 'JieYuan'
__mtime__ = '19-1-3'
"""

import autokeras as ak

# ak.MlpModule
#
# ak.ImageClassifier
# ak.ImageRegressor
#
# ak.TabularClassifier
# ak.TabularRegressor
# ak.TextClassifier
# ak.TextRegressor