#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : SavedModel
# @Time         : 2019-06-24 16:24
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


# path = '/tmp/simple_keras_model'
# tf.keras.experimental.export_saved_model(model, path)
#
# # Load the saved keras model back.
# new_model = tf.keras.experimental.load_from_saved_model(path)
# new_model.summary()
