#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : colors
# @Time         : 2019-07-04 14:03
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


class Colors:
    ok = '\033[92m'
    fail = '\033[91m'
    close = '\033[0m'



import csv

print(csv.QUOTE_NONE)