#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'fstring'
__author__ = 'JieYuan'
__mtime__ = '2019-05-06'
"""

'''https://blog.csdn.net/sunxb10/article/details/81036693'''
'''https://docs.python.org/3/library/string.html#format-string-syntax'''
# %[(name)][flags][width].[precision]typecode
