#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : __init__.py
# @Time         : 2019-06-21 14:43
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from .TextFast import TextFast
from .TextCNN import TextCNN
from .TextRNN import TextRNN
from .TextRNNBi import TextRNNBi
from .TextRCNN import TextRCNN
from .TextRCNNVariant import TextRCNNVariant
