#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : pandas
# @Time         : 2019-06-24 13:20
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


# https://github.com/tensorflow/tensorflow/issues/20995
import tensorflow as tf

# input_func = tf.estimator.inputs.pandas_input_fn()
#
#
# tf.estim